if not _G[...] then return end
local addon = _G[...]
local addonName = addon.addonName

local L = addon.L

--[[-----------------------------------------------------------------------------
Helpers
-------------------------------------------------------------------------------]]
local function Get(info)
	return addon.db.profile[info[#info]]
end

local function GetColor(info)
	local color = addon.db.profile[info[#info]]
	return color[1], color[2], color[3], color[4]
end

local function HideOptions(info)
	local field = info[#info]
	if addon.db.profile.independent then
		return field == "spacing" or field == "squaring"
	else
		return field == "spacingX" or field == "spacingY"
	end
end

local function Set(info, value)
	addon.db.profile[info[#info]] = value
	if info[#info] == "centerThickness" then
		addon.UpdateCenterLines()
		if addon.db.profile.absolute then return end
	end
	addon.DrawGrid()
end

local function SetColor(info, ...)
	local field = info[#info]
	local color = addon.db.profile[field]
	color[1], color[2], color[3], color[4] = ...
	addon.ApplyColor(field)
end

local function SetState(info, value)
	addon.db.profile[info[#info]] = value
	addon.UpdateState()
end

--[[-----------------------------------------------------------------------------
Options
-------------------------------------------------------------------------------]]
local options = {
	type = 'group', childGroups = 'tab',
	args = {
		general = {
			type = 'group',
			name = L["General"],
			desc = L["General settings."],
			get = Get,
			set = Set,
			args = {
				hide = {
					type = 'toggle',
					order = 1,
					name = L["Hide"],
					desc = L["Remove the grid."],
					set = SetState
				},
				overlay = {
					type = 'toggle',
					order = 2,
					name = L["Overlay"],
					desc = L["Move the grid from the background to the foreground."],
					set = SetState
				},
				window = {
					type = 'select',
					order = 3,
					name = L["Window"],
					desc = L["Set the window that the grid will be applied to."],
					values = { L["Entire Screen"], L["User Interface"], L["3D World"] },
					set = SetState
				},
				centerHeader = {
					type = 'header',
					order = 10,
					name = L["Center Lines"],
				},
				centerColor = {
					type = 'color',
					order = 11,
					name = L["Color"],
					desc = L["Change the color of the center lines."],
					hasAlpha = true,
					get = GetColor,
					set = SetColor
				},
				centerThickness = {
					type = 'range',
					order = 12,
					name = L["Size"],
					desc = L["Change the thickness of the center lines."],
					min = 1, max = 3, step = 1
				},
				gridHeader = {
					type = 'header',
					order = 20,
					name = L["Grid Lines"],
				},
				gridColor = {
					type = 'color',
					order = 21,
					name = L["Color"],
					desc = L["Change the color of the grid lines."],
					hasAlpha = true,
					get = GetColor,
					set = SetColor
				},
				gridThickness = {
					type = 'range',
					order = 22,
					name = L["Size"],
					desc = L["Change the thickness of the grid lines."],
					min = 1, max = 3, step = 1
				},
				spacingHeader = {
					type = 'header',
					order = 30,
					name = L["Line Spacing"],
				},
				absolute = {
					type = 'toggle',
					order = 31,
					name = L["Absolute"],
					desc = L["Grid lines are spaced without regard for line thickness."]
				},
				independent = {
					type = 'toggle',
					order = 32,
					name = L["Independent"],
					desc = L["Adjust the spacing for grid lines seperately for each axis."],
					set = function(info, value)
						local settings = addon.db.profile
						settings[info[#info]] = value
						if value then
							settings.spacingY = settings.spacingX
						end
						addon.DrawGrid()
					end
				},
				spacing = {
					type = 'range',
					order = 33,
					name = L["Spacing"],
					desc = L["Adjust the distance between grid lines."],
					hidden = HideOptions,
					min = addon.MIN_SPACING, max = addon.MAX_SPACING, step = 1,
					get = function(info)
						return addon.db.profile.spacingX
					end,
					set = function(info, value)
						addon.db.profile.spacingX = value
						addon.DrawGrid()
					end
				},
				spacingX = {
					type = 'range',
					order = 33,
					name = L["X-Axis Spacing"],
					desc = L["Adjust the distance between grid lines along the X-axis."],
					hidden = HideOptions,
					min = addon.MIN_SPACING, max = addon.MAX_SPACING, step = 1
				},
				spacingY = {
					type = 'range',
					order = 34,
					name = L["Y-Axis Spacing"],
					desc = L["Adjust the distance between grid lines along the Y-axis."],
					hidden = HideOptions,
					min = addon.MIN_SPACING, max = addon.MAX_SPACING, step = 1
				},
				squaring = {
					type = 'toggle',
					order = 34,
					name = L["Squaring"],
					desc = L["Make the grid lines intersect to form squares."],
					hidden = HideOptions
				},
				backdropHeader = {
					type = 'header',
					order = 40,
					name = L["Backdrop"],
				},
				backdropEnable = {
					type = 'toggle',
					order = 41,
					name = L["Enable"],
					desc = function()
						return addon.db.profile.backdropIndependent and L["Allow the backdrop to be shown."] or L["Allow the backdrop to be shown with the grid."]
					end,
					set = SetState
				},
				backdropColor = {
					type = 'color',
					order = 42,
					name = L["Color"],
					desc = L["Change the color of the backdrop."],
					hasAlpha = true,
					get = GetColor,
					set = SetColor
				},
				backdropIndependent = {
					type = 'toggle',
					order = 43,
					name = L["Independent"],
					desc = L["Allow the backdrop to be shown while the grid is hidden."],
					set = SetState
				}
			}
		}
	}
}

--[[-----------------------------------------------------------------------------
Initialize
-------------------------------------------------------------------------------]]
LibStub('AceConfigRegistry-3.0'):RegisterOptionsTable(addonName, options)

local profileOptions = LibStub('AceDBOptions-3.0'):GetOptionsTable(addon.db)
options.args.profiles = profileOptions
profileOptions.order = -1

addon.config:AssignOptions(addonName)
addon.config:SetDesc(L["These options allow you to change the appearance of %s."]:format(addonName))
addon.config:SetInfo(L["Version"] .. ": " .. GetAddOnMetadata(addonName, 'Version'))
